#include QMK_KEYBOARD_H
#include "debug.h"
#include "action_layer.h"

#define BASE 0 // default layer
#define CODE 1 // code chars | move layer
#define KYPD 2 // accents    | keypad layer

#define CAPS_CTL CTL_T(KC_CAPS) // Caps on tap, Ctrl on hold.

#define M_COPY   LCTL(KC_INS) // Ctrl  - Ins   Copy
#define M_CUT    LSFT(KC_DEL) // Shift - Suppr Cut
#define M_PASTE  LSFT(KC_INS) // Shift - Ins   Paste

#define C_COPY   LCTL(KC_C) // C-c Copy
#define C_CUT    LCTL(KC_X) // C-x Cut
#define C_PASTE  LCTL(KC_V) // C-v Paste

#define _OTAG  1 // <>
#define _CTAG  2 // </>
#define _NBSP  3 // </>
#define _HQF   4 // Sign. = Olivier Pons / HQF Development
#define _GT    5 // >  Greater than
#define _PPE   6 // |  Pipe
#define _BKS   7 // \\ Backslash
#define _LBK   8 // { Left bracket
#define _RBK   9 // { Right bracket
#define _SLH  10 // / Slash
#define _HTG  11 // # Hashtag
#define _OSB  12 // [ Opening Square bracket
#define _CSB  13 // ] Closing Square bracket
#define _ARB  14 // @ Arobase
#define _CTL  15 // Ctrl ←
#define _CTR  16 // Ctrl →
#define _TLD  17 // ~ Tilde
#define _BQT  18 // ` Backquote

// Special KEYMAPPING for easy readability:
#define ERGODOX_KEYMAP( \
  L01,L02,L03,L04,L05,L06,L07,    R01,R02,R03,R04,R05,R06,R07, \
  L08,L09,L10,L11,L12,L13,L14,    R08,R09,R10,R11,R12,R13,R14, \
  L15,L16,L17,L18,L19,L20,            R15,R16,R17,R18,R19,R20, \
  L21,L22,L23,L24,L25,L26,L27,    R21,R22,R23,R24,R25,R26,R27, \
  L28,L29,L30,L31,L32,                    R28,R29,R30,R31,R32, \
                       L33,L34,   R33,R34, \
                           L35,   R35, \
                   L36,L37,L38,   R36,R37,R38 \
  ) \
  LAYOUT_ergodox( \
      (L01),(L02),(L03),(L04),(L05),(L06),(L07),(L08),(L09),(L10),(L11) , \
      (L12),(L13),(L14),(L15),(L16),(L17),(L18),(L19),(L20),(L21),(L22) , \
      (L23),(L24),(L25),(L26),(L27),(L28),(L29),(L30),(L31),(L32),(L33) , \
      (L34),(L35),(L36),(L37),(L38),(R01),(R02),(R03),(R04),(R05),(R06) , \
      (R07),(R08),(R09),(R10),(R11),(R12),(R13),(R14),(R15),(R16),(R17) , \
      (R18),(R19),(R20),(R21),(R22),(R23),(R24),(R25),(R26),(R27),(R28) , \
      (R29),(R30),(R31),(R32),(R33),(R34),(R35),(R36),(R37),(R38) \
  )

// NOTE: Cells marked with ACCESS must remain transparent, they're the keys that actually get to that layer


const uint16_t PROGMEM keymaps[][MATRIX_ROWS][MATRIX_COLS] = {
[BASE] = ERGODOX_KEYMAP( // Layer 0 :
  /* Left and right: (key) and (shifted key)                                                                                     */
  /* ,----------------------------------------------------------.   ,----------------------------------------------------------. */
  /* |   Esc   |   & 1 |   é 2 |   " 3 |   ' 4 |   ( 5 |   ^L3  |   |  Pscr  |   - 6 |   è 7 |   _ 8 |   ç 9 |   à 0 |  Bspc   | */
       KC_ESC  , KC_1  , KC_2  , KC_3  , KC_4  , KC_5  ,  KC_A  ,    KC_PSCR , KC_6  , KC_7  , KC_8  , KC_9  , KC_0  , KC_BSPC ,
  /* |---------+-------+-------+-------+-------+----------------|   |--------+-------+-------+-------+-------+-------+---------| */
  /* |   Tab   |   a A |   z Z |   e E |   r R |   t T |  Bspc  |   |    +   |   y Y |   u U |   i I |   o O |   p P |   ^ ¨   | */
       KC_TAB  , KC_Q  , KC_W  , KC_E  , KC_R  , KC_T  , KC_BSPC,    KC_PPLS , KC_Y  , KC_U  , KC_I  , KC_O  , KC_P  , KC_LBRC ,
  /* |---------+-------+-------+-------+-------+-------|        |   |        |-------+-------+-------+-------+-------+---------| */
  /* |    .    |   q Q |   s S |   d D |   f F |   g G |--------|   |--------|   h H |   j J |   k K |   l L |   m M |  Enter  | */
       KC_PDOT , KC_A  , KC_S  , KC_D  , KC_F  , KC_G  ,                       KC_H  , KC_J  , KC_K  , KC_L  ,KC_SCLN, KC_ENT  ,
  /* |---------+-------+-------+-------+-------+-------|   Del  |   |    -   |-------+-------+-------+-------+-------+---------| */
  /* |  LSft   |   w W |   x X |   c C |   v V |   b B |        |   |        |   n N |   , ? |   ; . |   : / |   ! § |  RSft   | */
       KC_LSFT , KC_Z  , KC_X  , KC_C  , KC_V  , KC_B  , KC_DEL ,    KC_PMNS , KC_N  , KC_M  ,KC_COMM,KC_DOT ,KC_SLASH, KC_RSFT ,
  /* `---------+-------+-------+-------+-------+----------------'   `----------------+-------+-------+-------+--------+--------' */
  /*   | LCtl  |  LGui |  LAlt |  Bspc |   Del |                                   |  Spc  |  RSft |  RAlt |  RGui  |  RCtl  |   */
        KC_LCTL,KC_LGUI,KC_LALT,KC_BSPC,KC_DEL ,                                    KC_SPC ,KC_RSFT,KC_RALT,KC_RGUI , KC_RCTL,
  /*   `---------------------------------------'                                   `-----------------------------------------'   */
  /*                                         ,------------------.   ,------------------.                                          */
  /*                                         |  Copy   |  Pste  |   |    <>  |   ^L5   |                                          */
                                               M_COPY  ,M_PASTE ,    M(_OTAG), KC_SPC  ,
  /*                               ,---------|---------|--------|   |--------+---------+---------.                                */
  /*                               |         |         |   Cut  |   |   </>  |         |         |                                */
                                                          M_CUT ,    M(_CTAG),
  /*                               |   ~L1   |   ~L2   |--------|   |--------|    ~L2  |   ~L1   |                                */
  /*                               |         |         |   Del  |   |  Nbsp  |         |         |                                */
                                    MO(CODE) , MO(KYPD), KC_DEL ,    M(_NBSP), MO(KYPD), MO(CODE)
  /*                               `----------------------------'   `----------------------------'                                */
  ),

[CODE] = ERGODOX_KEYMAP( // Layer 1
  /* Left hand = chars for developper                               Right hand = moving cursor + cut/copy/paste                  */
  /* ,----------------------------------------------------------.   ,----------------------------------------------------------. */
  /* |   Esc   |   F1  |   F2  |   F3  |   F4  |   F5  |   F6   |   |   F7   |   F8  |   F9  |  F10  |  F11  |  F12  |  Bspc   | */
       KC_TRNS , KC_F1 , KC_F2 , KC_F3 , KC_F4 , KC_F5 ,  KC_F6 ,     KC_F7  , KC_F8 , KC_F9 , KC_F10, KC_F11, KC_F12, KC_TRNS ,
  /* |---------+-------+-------+-------+-------+----------------|   |--------+-------+-------+-------+-------+-------+---------| */
  /* |   Tab   |   \   |   <   |   _   |   >   |   |   |   $    |   |  Sign. | Copy  | Ct-←  |   ↑   | Ct-→  | Pste  |   ^ ¨   | */
       KC_TRNS ,M(_BKS),KC_NUBS, KC_8  ,M(_GT) ,M(_PPE), KC_RBRC,     M(_HQF), M_COPY,M(_CTL),KC_TRNS,M(_CTR),M_PASTE, KC_LBRC ,
  /* |---------+-------+-------+-------+-------+-------|        |   |        |-------+-------+-------+-------+-------+---------| */
  /* |     -   |   {   |   (   |   /   |   )   |   }   |--------|   |--------|  Home |   ←   |   ↓   |   →   |  End  |  Enter  | */
       KC_TRNS ,M(_LBK), KC_5  ,M(_SLH),KC_MINS,M(_RBK),                      KC_HOME,KC_LEFT,KC_DOWN,KC_RGHT,KC_END , KC_TRNS ,
  /* |---------+-------+-------+-------+-------+-------|   =    |   |    -   |-------+-------+-------+-------+-------+---------| */
  /* |  Lsft   |   #   |   [   |   *   |   ]   |   @   |        |   |        |   -   | PgUp  |  Ins  | PgDn  | PEnt  |  RSft   | */
       KC_TRNS ,M(_HTG),M(_OSB),KC_BSLS,M(_CSB),M(_ARB), KC_EQL ,    KC_TRNS ,KC_TRNS,KC_PGUP,KC_INS,KC_PGDN,KC_PENT , KC_TRNS ,
  /* `---------+-------+-------+-------+-------+----------------'   `----------------+-------+-------+-------+-------+---------' */
  /*   | LCtl  |  LGui |  LAlt |  Bspc |   Del |                                   |   0   |   .   |  RAlt |  RGui |  RCtl |     */
        KC_TRNS,KC_TRNS,KC_TRNS,KC_TRNS,KC_TRNS,                                     KC_P0 ,KC_DOT ,KC_TRNS,KC_TRNS,KC_TRNS,
  /*   `---------------------------------------'                                   `---------------------------------------'     */
  /*                                          ,-----------------.   ,-----------------.                                          */
  /*                                          | C-Copy | C-Pste |   |    <>  |   ^L5  |                                          */
                                               C_COPY  ,C_PASTE ,     KC_TRNS ,KC_TRNS ,
  /*                                   ,------|--------|--------|   |--------+--------+------.                                   */
  /*                                   |      |        | C-Cut  |   |   </>  |        |      |                                   */
                                                          C_CUT ,    KC_TRNS,
  /*                                   |  ~L1 |   ~L2  |--------|   |--------|   ~L2  |  ~L1 |                                   */
  /*                                   |      |        |   Del  |   |  Nbsp  |        |      |                                   */
                                        KC_TRNS,KC_TRNS,KC_TRNS ,    KC_TRNS, KC_TRNS ,KC_TRNS
  /*                                   `------------------------'   `------------------------'                                   */
  ),

[KYPD] = ERGODOX_KEYMAP( // Layer 2
  /* Left hand = accents                                             Right hand = keypad and caps / num lock                     */
  /* ,----------------------------------------------------------.   ,----------------------------------------------------------. */
  /* |    -    |   -   |   -   |   -   |   -   |   -   |   F6   |   |  CAPS  |  NLCK |  NLCK |   /   |   *   |   -   |  Bspc   | */
       KC_TRNS ,KC_TRNS,KC_TRNS,KC_TRNS,KC_TRNS,KC_TRNS, KC_TRNS,    KC_CAPS ,KC_NLCK,KC_NLCK,KC_PSLS,KC_PAST,KC_PMNS, KC_TRNS ,
  /* |---------+-------+-------+-------+-------+----------------|   |--------+-------+-------+-------+-------+-------+---------| */
  /* |    -    |   -   |   -   |   -   |   -   |   -   |   ~    |   |  Sign. | Copy  |   7   |   8   |   9   |   +   |   ^ ¨   | */
       KC_TRNS ,KC_TRNS,KC_TRNS,KC_TRNS,KC_TRNS,KC_TRNS, M(_TLD),     M(_HQF),KC_TRNS, KC_P7 , KC_P8 , KC_P9 ,KC_PPLS, KC_TRNS ,
  /* |---------+-------+-------+-------+-------+-------|        |   |        |-------+-------+-------+-------+-------+---------| */
  /* |    -    |   -   |   -   |   -   |   -   |   -   |--------|   |--------|  Home |   4   |   5   |   6   |   +   |  Enter  | */
       KC_TRNS ,KC_TRNS,KC_TRNS,KC_TRNS,KC_TRNS,KC_TRNS,                      KC_HOME, KC_P4 , KC_P5 , KC_P6 ,KC_PPLS, KC_TRNS ,
  /* |---------+-------+-------+-------+-------+-------|   `    |   |    -   |-------+-------+-------+-------+-------+---------| */
  /* |    -    |   -   |   -   |   -   |   -   |   -   |        |   |        |   -   |   1   |   2   |   3   | PEnt  |  RSft   | */
       KC_TRNS ,KC_TRNS,KC_TRNS,KC_TRNS,KC_TRNS,KC_TRNS, M(_BQT),    KC_TRNS ,KC_TRNS, KC_P1 , KC_P2 , KC_P3 ,KC_PENT, KC_TRNS ,
  /* `---------+-------+-------+-------+-------+----------------'   `----------------+-------+-------+-------+-------+---------' */
  /*   |   -   |   -   |   -   |   -   |   -   |                                   |   0   |   .   |  RAlt |  RGui |  RCtl |     */
        KC_TRNS,KC_TRNS,KC_TRNS,KC_TRNS,KC_TRNS,                                     KC_P0 ,KC_DOT ,KC_TRNS,KC_TRNS,KC_TRNS,
  /*   `---------------------------------------'                                   `---------------------------------------'     */
  /*                                          ,-----------------.   ,-----------------.                                          */
  /*                                          |  Copy  |  Pste  |   |    <>  |   ^L5  |                                          */
                                               KC_TRNS ,KC_TRNS,     KC_TRNS ,KC_TRNS ,
  /*                                   ,------|--------|--------|   |--------+--------+------.                                   */
  /*                                   |      |        |   Cut  |   |   </>  |        |      |                                   */
                                                         KC_TRNS,    KC_TRNS,
  /*                                   |  ~L1 |   ~L2  |--------|   |--------|   ~L2  |  ~L1 |                                   */
  /*                                   |      |        |   Del  |   |  Nbsp  |        |      |                                   */
                                        KC_TRNS,KC_TRNS,KC_TRNS ,    KC_TRNS, KC_TRNS ,KC_TRNS
  /*                                   `------------------------'   `------------------------'                                   */
  ),
};

const uint16_t PROGMEM fn_actions[] = {
};

const macro_t *action_get_macro(keyrecord_t *record, uint8_t id, uint8_t opt)
{
    // MACRODOWN only works in this function
    switch(id) {
        case _OTAG: // Opening tag <>
            return MACRODOWN( \
                T(NUBS), D(RSFT), T(NUBS), U(RSFT), T(LEFT), \
                END);
        case _CTAG: // Closing tag </>
            return MACRODOWN( \
                T(NUBS), D(RSFT), T(DOT), T(NUBS), U(RSFT), T(LEFT), \
                END);
        case _NBSP: // &nbsp;
            return MACRODOWN( \
                T(1), T(N), T(B), T(S), T(P), T(COMM), \
                END);
        case _HQF: // Olivier Pons / HQF Development
            return MACRODOWN( T(ENT), \
                DOWN(KC_LSFT), T(O), UP(KC_LSFT),
				T(L), T(I), T(V), T(I), T(E), T(R), T(SPC), \
                DOWN(KC_LSFT), T(P), UP(KC_LSFT), T(O), T(N), T(S), T(SPC), \
                DOWN(KC_LSFT), T(DOT), UP(KC_LSFT), T(SPC), \
                DOWN(KC_LSFT), T(H), T(A), T(F), UP(KC_LSFT), T(SPC), \
                DOWN(KC_LSFT), T(D), UP(KC_LSFT), T(E), T(V), T(E), T(L), \
                T(O), T(P), T(SCLN), T(E), T(N), T(T), \
                END);
        case _GT: // > Greater than
            return MACRODOWN( D(RSFT), T(NUBS), U(RSFT), END);
        case _PPE: // | Pipe
            return MACRODOWN( D(RALT), T(6), U(RALT), END);
        case _BKS: // \ Backslash
            return MACRODOWN( D(RALT), T(8), U(RALT), END);
        case _LBK: // { Left bracket
            return MACRODOWN( D(RALT), T(4), U(RALT), END);
        case _RBK: // } Right bracket
            return MACRODOWN( D(RALT), T(EQL), U(RALT), END);
        case _SLH: // / Slash
            return MACRODOWN( D(RSFT), T(DOT), U(RSFT), END);
        case _HTG: // # Hashtag
            return MACRODOWN( D(RALT), T(3), U(RALT), END);
        case _OSB: // [ Opening Square bracket
            return MACRODOWN( D(RALT), T(5), U(RALT), END);
        case _CSB: // ] Closing Square bracket
            return MACRODOWN( D(RALT), T(MINS), U(RALT), END);
        case _ARB: // @ Arobase
            return MACRODOWN( D(RALT), T(0), U(RALT), END);
        case _CTL: // Ctrl ←
            return MACRODOWN( D(LCTL), T(LEFT), U(LCTL), END);
        case _CTR: // Ctrl →
            return MACRODOWN( D(LCTL), T(RIGHT), U(LCTL), END);
        case _TLD: // ~ Tilde
            return MACRODOWN( D(RALT), T(2), U(RALT), END);
        case _BQT: // ` Backquote
            return MACRODOWN( D(RALT), T(7), T(7), U(RALT), END);
    };
    return MACRO_NONE;
};

// Runs just one time when the keyboard initializes.
void matrix_init_user(void) {

};

// Runs constantly in the background, in a loop.
void matrix_scan_user(void) {

    uint8_t layer = biton32(layer_state);

    ergodox_board_led_off();
    ergodox_right_led_1_off();
    ergodox_right_led_2_off();
    ergodox_right_led_3_off();
    switch (layer) {
        case 1:
            ergodox_right_led_1_on();
            break;
        case 2:
            ergodox_right_led_2_on();
            break;
        default:
        // none
        break;
    }

};
